/*********************************************************************
 *
 * $Id: YDataLogger.cpp 14508 2014-01-16 13:37:02Z martinm $
 *
 * Implements commands to handle DataLogger functions
 *
 * - - - - - - - - - License information: - - - - - - - - - 
 *
 * Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
 *
 * 1) If you have obtained this file from www.yoctopuce.com,
 *    Yoctopuce Sarl licenses to you (hereafter Licensee) the
 *    right to use, modify, copy, and integrate this source file
 *    into your own solution for the sole purpose of interfacing
 *    a Yoctopuce product with Licensee's solution.
 *
 *    The use of this file and all relationship between Yoctopuce 
 *    and Licensee are governed by Yoctopuce General Terms and 
 *    Conditions.
 *
 *    THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
 *    WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING 
 *    WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS 
 *    FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
 *    EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
 *    INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, 
 *    COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR 
 *    SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT 
 *    LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
 *    CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
 *    BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
 *    WARRANTY, OR OTHERWISE.
 *
 * 2) If your intent is not to interface with Yoctopuce products,
 *    you are not entitled to use, read or create any derived
 *    material from this source file.
 *
 *********************************************************************/


#include <stdio.h>
#include "cmdline_common.h"
#include "yocto_api.h"
#include "yocto_datalogger.h"

using namespace std;

//--- (generated code: YDataLogger definitions)

const char *enumBool[] = {
    "FALSE", 
    "TRUE", 
    NULL
};

const char *enumOnOff[] = {
    "OFF", 
    "ON", 
    NULL
};

//--- (end of generated code: YDataLogger definitions)

bool   isModule(void)      { return false; }

//--- (generated code: YDataLogger implementation)
string YFunctionName(void) { return "DataLogger"; }

// type of logicalName is string (Text)
// type of advertisedValue is string (PubText)
// type of currentRunIndex is int (UInt31)
// type of timeUTC is long (UTCTime)
// type of recording is int (OnOff)
// type of autoStart is int (OnOff)
// type of clearHistory is int (Bool)
/**
 * Returns the logical name of the data logger.
 * 
 * @return a string corresponding to the logical name of the data logger
 * 
 * On failure, throws an exception or returns Y_LOGICALNAME_INVALID.
 */
class DataLogger_get_logicalName : public YapiCommand /* arguments: */
{
public:

  string getName()
  {
    return "get_logicalName";
  }

  string getDescription()
  {
    return "Returns the logical name of the data logger.";
  }

  void execute(string target,vector<YModule*> *modulelist,string resultformat, vector<ArgumentDesc*>* args, vector<SwitchDesc*>* switches )
  {
    vector<YDataLogger*>* list = enumerateTargets<YDataLogger>(target,modulelist);
    unsigned int i;
    for (i = 0; i < list->size(); i++)
      {
        string attrval = (*list)[i]->get_logicalName();
        string strval =  attrval;
        PrintResult<YDataLogger>(resultformat, this->getName(),(*list)[i], strval, true);
      }
  }
};

/**
 * Changes the logical name of the data logger. You can use yCheckLogicalName()
 * prior to this call to make sure that your parameter is valid.
 * Remember to call the saveToFlash() method of the module if the
 * modification must be kept.
 * 
 * @param newval : a string corresponding to the logical name of the data logger
 * 
 * @return YAPI_SUCCESS if the call succeeds.
 * 
 * On failure, throws an exception or returns a negative error code.
 */
class DataLogger_set_logicalName : public YapiCommand /* arguments: newval */
{
public:

  string getName()
  {
    return "set_logicalName";
  }

  string getDescription()
  {
    return "Changes the logical name of the data logger.";
  }

  string getMoreInfo()
  {
    return "You can use yCheckLogicalName() prior to this call to make sure that your parameter is valid. Remember to call \"YModule {target} saveToFlash\" if the modification must be kept.";
  }

  vector<ArgumentDesc*>* getArgumentDesc()
  {
    vector<ArgumentDesc*>* res = new vector<ArgumentDesc*>();
    res->push_back(new ArgumentDesc(STRING_ARG, "newval", "a string corresponding to the logical name of the data logger", "_LOGICAL", false));
    return res;
  }

  void execute(string target,vector<YModule*> *modulelist,string resultformat, vector<ArgumentDesc*>* args , vector<SwitchDesc*>* switches)
  {
    vector<YDataLogger*>* list = enumerateTargets<YDataLogger>(target,modulelist);
    unsigned int i;
    string value = (*args)[0]->to_str();
    for (i = 0; i < list->size(); i++)
      {
        (*list)[i]->set_logicalName(string((*args)[0]->get_stringValue()));
        PrintResult<YDataLogger>(resultformat, this->getName(),(*list)[i], value, true);
      }
  }
};

/**
 * Returns the current value of the data logger (no more than 6 characters).
 * 
 * @return a string corresponding to the current value of the data logger (no more than 6 characters)
 * 
 * On failure, throws an exception or returns Y_ADVERTISEDVALUE_INVALID.
 */
class DataLogger_get_advertisedValue : public YapiCommand /* arguments: */
{
public:

  string getName()
  {
    return "get_advertisedValue";
  }

  string getDescription()
  {
    return "Returns the current value of the data logger (no more than 6 characters).";
  }

  void execute(string target,vector<YModule*> *modulelist,string resultformat, vector<ArgumentDesc*>* args, vector<SwitchDesc*>* switches )
  {
    vector<YDataLogger*>* list = enumerateTargets<YDataLogger>(target,modulelist);
    unsigned int i;
    for (i = 0; i < list->size(); i++)
      {
        string attrval = (*list)[i]->get_advertisedValue();
        string strval =  attrval;
        PrintResult<YDataLogger>(resultformat, this->getName(),(*list)[i], strval, true);
      }
  }
};

/**
 * Returns the current run number, corresponding to the number of times the module was
 * powered on with the dataLogger enabled at some point.
 * 
 * @return an integer corresponding to the current run number, corresponding to the number of times the module was
 *         powered on with the dataLogger enabled at some point
 * 
 * On failure, throws an exception or returns Y_CURRENTRUNINDEX_INVALID.
 */
class DataLogger_get_currentRunIndex : public YapiCommand /* arguments: */
{
public:

  string getName()
  {
    return "get_currentRunIndex";
  }

  string getDescription()
  {
    return "Returns the current run number, corresponding to the number of times the module waspowered on with the dataLogger enabled at some point.";
  }

  void execute(string target,vector<YModule*> *modulelist,string resultformat, vector<ArgumentDesc*>* args, vector<SwitchDesc*>* switches )
  {
    vector<YDataLogger*>* list = enumerateTargets<YDataLogger>(target,modulelist);
    unsigned int i;
    for (i = 0; i < list->size(); i++)
      {
        int attrval = (*list)[i]->get_currentRunIndex();
        char buf[32]; snprintf(buf, 32,"%d", attrval); string strval =  string(buf);
        PrintResult<YDataLogger>(resultformat, this->getName(),(*list)[i], strval, true);
      }
  }
};

/**
 * Returns the Unix timestamp for current UTC time, if known.
 * 
 * @return an integer corresponding to the Unix timestamp for current UTC time, if known
 * 
 * On failure, throws an exception or returns Y_TIMEUTC_INVALID.
 */
class DataLogger_get_timeUTC : public YapiCommand /* arguments: */
{
public:

  string getName()
  {
    return "get_timeUTC";
  }

  string getDescription()
  {
    return "Returns the Unix timestamp for current UTC time, if known.";
  }

  void execute(string target,vector<YModule*> *modulelist,string resultformat, vector<ArgumentDesc*>* args, vector<SwitchDesc*>* switches )
  {
    vector<YDataLogger*>* list = enumerateTargets<YDataLogger>(target,modulelist);
    unsigned int i;
    for (i = 0; i < list->size(); i++)
      {
        long attrval = (*list)[i]->get_timeUTC();
        char buf[64]; snprintf(buf, 64, "%u [s] since Jan 1, 1970", (u32)attrval); string strval =  string(buf);
        PrintResult<YDataLogger>(resultformat, this->getName(),(*list)[i], strval, true);
      }
  }
};

/**
 * Changes the current UTC time reference used for recorded data.
 * 
 * @param newval : an integer corresponding to the current UTC time reference used for recorded data
 * 
 * @return YAPI_SUCCESS if the call succeeds.
 * 
 * On failure, throws an exception or returns a negative error code.
 */
class DataLogger_set_timeUTC : public YapiCommand /* arguments: newval */
{
public:

  string getName()
  {
    return "set_timeUTC";
  }

  string getDescription()
  {
    return "Changes the current UTC time reference used for recorded data.";
  }

  vector<ArgumentDesc*>* getArgumentDesc()
  {
    vector<ArgumentDesc*>* res = new vector<ArgumentDesc*>();
    res->push_back(new ArgumentDesc(LONG_ARG, "newval", "an integer corresponding to the current UTC time reference used for recorded data", "_LONG", false));
    return res;
  }

  void execute(string target,vector<YModule*> *modulelist,string resultformat, vector<ArgumentDesc*>* args , vector<SwitchDesc*>* switches)
  {
    vector<YDataLogger*>* list = enumerateTargets<YDataLogger>(target,modulelist);
    unsigned int i;
    string value = (*args)[0]->to_str();
    for (i = 0; i < list->size(); i++)
      {
        (*list)[i]->set_timeUTC(long((*args)[0]->get_longValue()));
        PrintResult<YDataLogger>(resultformat, this->getName(),(*list)[i], value, true);
      }
  }
};

/**
 * Returns the current activation state of the data logger.
 * 
 * @return either Y_RECORDING_OFF or Y_RECORDING_ON, according to the current activation state of the data logger
 * 
 * On failure, throws an exception or returns Y_RECORDING_INVALID.
 */
class DataLogger_get_recording : public YapiCommand /* arguments: */
{
public:

  string getName()
  {
    return "get_recording";
  }

  string getDescription()
  {
    return "Returns the current activation state of the data logger.";
  }

  void execute(string target,vector<YModule*> *modulelist,string resultformat, vector<ArgumentDesc*>* args, vector<SwitchDesc*>* switches )
  {
    vector<YDataLogger*>* list = enumerateTargets<YDataLogger>(target,modulelist);
    unsigned int i;
    for (i = 0; i < list->size(); i++)
      {
        int attrval = (*list)[i]->get_recording();
        string strval =  enumString(attrval,enumOnOff);
        PrintResult<YDataLogger>(resultformat, this->getName(),(*list)[i], strval, true);
      }
  }
};

/**
 * Changes the activation state of the data logger to start/stop recording data.
 * 
 * @param newval : either Y_RECORDING_OFF or Y_RECORDING_ON, according to the activation state of the
 * data logger to start/stop recording data
 * 
 * @return YAPI_SUCCESS if the call succeeds.
 * 
 * On failure, throws an exception or returns a negative error code.
 */
class DataLogger_set_recording : public YapiCommand /* arguments: newval */
{
public:

  string getName()
  {
    return "set_recording";
  }

  string getDescription()
  {
    return "Changes the activation state of the data logger to start/stop recording data.";
  }

  vector<ArgumentDesc*>* getArgumentDesc()
  {
    vector<ArgumentDesc*>* res = new vector<ArgumentDesc*>();
    res->push_back(new ArgumentDesc(INT_ARG, "newval", "either OFF or ON, according to the activation state of the data logger to start/stop recording data", "OFF=0,ON=1", false));
    return res;
  }

  void execute(string target,vector<YModule*> *modulelist,string resultformat, vector<ArgumentDesc*>* args , vector<SwitchDesc*>* switches)
  {
    vector<YDataLogger*>* list = enumerateTargets<YDataLogger>(target,modulelist);
    unsigned int i;
    string value = (*args)[0]->to_str();
    for (i = 0; i < list->size(); i++)
      {
        (*list)[i]->set_recording(Y_RECORDING_enum((*args)[0]->get_intValue()));
        PrintResult<YDataLogger>(resultformat, this->getName(),(*list)[i], value, true);
      }
  }
};

/**
 * Returns the default activation state of the data logger on power up.
 * 
 * @return either Y_AUTOSTART_OFF or Y_AUTOSTART_ON, according to the default activation state of the
 * data logger on power up
 * 
 * On failure, throws an exception or returns Y_AUTOSTART_INVALID.
 */
class DataLogger_get_autoStart : public YapiCommand /* arguments: */
{
public:

  string getName()
  {
    return "get_autoStart";
  }

  string getDescription()
  {
    return "Returns the default activation state of the data logger on power up.";
  }

  void execute(string target,vector<YModule*> *modulelist,string resultformat, vector<ArgumentDesc*>* args, vector<SwitchDesc*>* switches )
  {
    vector<YDataLogger*>* list = enumerateTargets<YDataLogger>(target,modulelist);
    unsigned int i;
    for (i = 0; i < list->size(); i++)
      {
        int attrval = (*list)[i]->get_autoStart();
        string strval =  enumString(attrval,enumOnOff);
        PrintResult<YDataLogger>(resultformat, this->getName(),(*list)[i], strval, true);
      }
  }
};

/**
 * Changes the default activation state of the data logger on power up.
 * Remember to call the saveToFlash() method of the module if the
 * modification must be kept.
 * 
 * @param newval : either Y_AUTOSTART_OFF or Y_AUTOSTART_ON, according to the default activation state
 * of the data logger on power up
 * 
 * @return YAPI_SUCCESS if the call succeeds.
 * 
 * On failure, throws an exception or returns a negative error code.
 */
class DataLogger_set_autoStart : public YapiCommand /* arguments: newval */
{
public:

  string getName()
  {
    return "set_autoStart";
  }

  string getDescription()
  {
    return "Changes the default activation state of the data logger on power up.";
  }

  string getMoreInfo()
  {
    return "Remember to call \"YModule {target} saveToFlash\" if the modification must be kept.";
  }

  vector<ArgumentDesc*>* getArgumentDesc()
  {
    vector<ArgumentDesc*>* res = new vector<ArgumentDesc*>();
    res->push_back(new ArgumentDesc(INT_ARG, "newval", "either OFF or ON, according to the default activation state of the data logger on power up", "OFF=0,ON=1", false));
    return res;
  }

  void execute(string target,vector<YModule*> *modulelist,string resultformat, vector<ArgumentDesc*>* args , vector<SwitchDesc*>* switches)
  {
    vector<YDataLogger*>* list = enumerateTargets<YDataLogger>(target,modulelist);
    unsigned int i;
    string value = (*args)[0]->to_str();
    for (i = 0; i < list->size(); i++)
      {
        (*list)[i]->set_autoStart(Y_AUTOSTART_enum((*args)[0]->get_intValue()));
        PrintResult<YDataLogger>(resultformat, this->getName(),(*list)[i], value, true);
      }
  }
};

// type of valueCallbackDataLogger is YDataLoggerValueCallback* (YDataLoggerValueCallback)
/**
 * Clears the data logger memory and discards all recorded data streams.
 * This method also resets the current run index to zero.
 * 
 * @return YAPI_SUCCESS if the call succeeds.
 * 
 * On failure, throws an exception or returns a negative error code.
 */
class apifun_DataLogger_forgetAllDataStreams : public YapiCommand /* arguments: */
{
public:

  string getName()
  {
    return "forgetAllDataStreams";
  }

  string getDescription()
  {
    return "Clears the data logger memory and discards all recorded data streams.";
  }

  string getMoreInfo()
  {
    return "This method also resets the current run index to zero.";
  }

  vector<ArgumentDesc*>* getArgumentDesc()
  {
    vector<ArgumentDesc*>* res = new vector<ArgumentDesc*>();
    return res;
  }

  void execute(string target,vector<YModule*> *modulelist,string resultformat, vector<ArgumentDesc*>* args , vector<SwitchDesc*>* switches)
  {
    vector<YDataLogger*>* list = enumerateTargets<YDataLogger>(target,modulelist);
    unsigned int i;
    string value = "";
    for (i = 0; i < list->size(); i++)
      {
        (*list)[i]->forgetAllDataStreams();
        PrintResult<YDataLogger>(resultformat, this->getName(),(*list)[i], value, true);
      }
  }
};

/**
 * Returns a list of YDataSet objects that can be used to retrieve
 * all measures stored by the data logger.
 * 
 * This function only works if the device uses a recent firmware,
 * as YDataSet objects are not supported by firmwares older than
 * version 13000.
 * 
 * @return a list of YDataSet object.
 * 
 * On failure, throws an exception or returns an empty list.
 */
class apifun_DataLogger_get_dataSets : public YapiCommand /* arguments: */
{
public:

  string getName()
  {
    return "get_dataSets";
  }

  string getDescription()
  {
    return "Returns a list of YDataSet objects that can be used to retrieveall measures stored by the data logger.";
  }

  string getMoreInfo()
  {
    return "This function only works if the device uses a recent firmware, as YDataSet objects are not supported by firmwares older than version 13000.";
  }

  vector<ArgumentDesc*>* getArgumentDesc()
  {
    vector<ArgumentDesc*>* res = new vector<ArgumentDesc*>();
    return res;
  }

  void execute(string target,vector<YModule*> *modulelist,string resultformat, vector<ArgumentDesc*>* args , vector<SwitchDesc*>* switches)
  {
    vector<YDataLogger*>* list = enumerateTargets<YDataLogger>(target,modulelist);
    unsigned int i;
    for (i = 0; i < list->size(); i++)
      {
        vector<YDataSet> value = (*list)[i]->get_dataSets();
        PrintResult<YDataLogger>(resultformat, this->getName(),(*list)[i], value, true);
      }
  }
};

//--- (end of generated code: YDataLogger implementation)

//--- (generated code: DataLogger functions)
void RegisterCommands(vector<YapiCommand*>* cmdList)
  {
    RegisterGenericFunctions<YDataLogger>(cmdList);
    cmdList->push_back((YapiCommand*) (new DataLogger_get_logicalName()));
    cmdList->push_back((YapiCommand*) (new DataLogger_set_logicalName()));
    cmdList->push_back((YapiCommand*) (new DataLogger_get_advertisedValue()));
    cmdList->push_back((YapiCommand*) (new DataLogger_get_currentRunIndex()));
    cmdList->push_back((YapiCommand*) (new DataLogger_get_timeUTC()));
    cmdList->push_back((YapiCommand*) (new DataLogger_set_timeUTC()));
    cmdList->push_back((YapiCommand*) (new DataLogger_get_recording()));
    cmdList->push_back((YapiCommand*) (new DataLogger_set_recording()));
    cmdList->push_back((YapiCommand*) (new DataLogger_get_autoStart()));
    cmdList->push_back((YapiCommand*) (new DataLogger_set_autoStart()));
    cmdList->push_back((YapiCommand*) (new apifun_DataLogger_forgetAllDataStreams()));
    cmdList->push_back((YapiCommand*) (new apifun_DataLogger_get_dataSets()));
  }

//--- (end of generated code: DataLogger functions)
