  /*********************************************************************
 *
 * $Id: cmdline_common.cpp 7646 2012-09-13 15:05:29Z mvuilleu $
 *
 * implementation of cmdline  API generic class & functions
 *
 * - - - - - - - - - License information: - - - - - - - - -
 *
 * Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
 *
 * 1) If you have obtained this file from www.yoctopuce.com,
 *    Yoctopuce Sarl licenses to you (hereafter Licensee) the
 *    right to use, modify, copy, and integrate this source file
 *    into your own solution for the sole purpose of interfacing
 *    a Yoctopuce product with Licensee's solution.
 *
 *    The use of this file and all relationship between Yoctopuce
 *    and Licensee are governed by Yoctopuce General Terms and
 *    Conditions.
 *
 *    THE SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT
 *    WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
 *    WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
 *    FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
 *    EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
 *    INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
 *    COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
 *    SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
 *    LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
 *    CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
 *    BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
 *    WARRANTY, OR OTHERWISE.
 *
 * 2) If your intent is not to interface with Yoctopuce products,
 *    you are not entitled to use, read or create any derived
 *    material from this source file.
 *
 *********************************************************************/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <cfloat>
#include <algorithm>
#include <iostream>
#include <fstream>

#include "cmdline_common.h"



ArgumentDesc::ArgumentDesc(argTypes type, string name, string description, string acceptedValues, bool optional)
{
    this->name = name;
    this->description = description;
    this->optional = optional;
    this->type = type;
    this->acceptedValues = new vector<string*>();
    string* res = new string();
    for (unsigned int i = 0; i < acceptedValues.length(); i++) {
        unsigned char c = acceptedValues[i];
        if (c == ',') {
            if (res->compare("") != 0) 
				this->acceptedValues->push_back(res);
            res = new string();
        } else {
			(*res) += c;
		}
    }
    if (res->compare("") != 0) 
		this->acceptedValues->push_back(res);
}

ArgumentDesc::~ArgumentDesc()
{
}

string ArgumentDesc::to_str(void)
{
    char buffer[1024];
    char *p = buffer;
    size_t i;
    switch (this->type) {
#ifdef __64BITS__
    case LONG_ARG:
    case INT_ARG:
            snprintf(buffer, 256, "%ld", this->intValue);
            break;
    case UNSIGNED_ARG:
            snprintf(buffer, 256, "%lu", this->unsignedValue);
            break;
#else
    case LONG_ARG:
    case INT_ARG:
            snprintf(buffer, 256, "%lld", this->intValue);
            break;
    case UNSIGNED_ARG:
            snprintf(buffer, 256, "%llu", this->unsignedValue);
            break;
#endif
    case STRING_ARG:
            snprintf(buffer, 256, "%s", this->stringValue.c_str());
            break;
    case FLOATARR_ARG:
            buffer[0] = 0;
            for(i = 0; i < this->floatArrValue.size(); i++) {
                if(i > 0) *p++ = ',';
                snprintf(p, buffer+1024-p, "%f", this->floatArrValue[i]);
                while(*p) p++;
            }
            break;
    case DOUBLE_ARG:
            snprintf(buffer, 256, "%f", this->doubleValue);
            break;
    case BIN_ARG:
            snprintf(buffer, 256, "(binary buffer)");
            break;
    }
    return string(buffer);
}

s64 ArgumentDesc::get_longValue()
{
    if (this->type != LONG_ARG) throw std::string("Argument type is not LONG");
    return this->intValue;
}

s64 ArgumentDesc::get_intValue()
{
    if (this->type != INT_ARG) throw std::string("Argument type is not INTEGER");
    return this->intValue;
}

u64 ArgumentDesc::get_unsignedValue()
{
    if (this->type != UNSIGNED_ARG) throw std::string("Argument type is not UNSIGNED");
    return this->unsignedValue;
}

string ArgumentDesc::get_stringValue()
{
    if (this->type != STRING_ARG) throw std::string("Argument type is not STRING");
    return this->stringValue;
}

double ArgumentDesc::get_doubleValue()
{
    if (this->type != DOUBLE_ARG) throw std::string("Argument type is not DOUBLE");
    return this->doubleValue;
}

vector<double> ArgumentDesc::get_floatArrValue()
{
    if (this->type != FLOATARR_ARG) throw std::string("Argument type is not FLOATARR");
    return this->floatArrValue;
}

string ArgumentDesc::get_binValue()
{
    if (this->type != BIN_ARG) throw std::string("Argument type is not BINARY");
    return this->binValue;
}

void ArgumentDesc::set_longValue(s64 value)
{
    if (this->type != LONG_ARG) throw std::string("Argument type is not LONG");
    this->intValue = value;
}

void ArgumentDesc::set_intValue(s64 value)
{
    if (this->type != INT_ARG) throw std::string("Argument type is not INT");
    this->intValue = value;
}

void ArgumentDesc::set_unsignedValue(u64 value)
{
    if (this->type != UNSIGNED_ARG) throw std::string("Argument type is not UNSIGNED");
    this->unsignedValue = value;
}

void ArgumentDesc::set_stringValue(string value)
{
    if (this->type != STRING_ARG) throw std::string("Argument type is not STRING");
    this->stringValue = value;
}

void ArgumentDesc::set_doubleValue(double value)
{
    if (this->type != DOUBLE_ARG) throw std::string("Argument type is not DOUBLE");
    this->doubleValue = value;
}

void ArgumentDesc::set_floatArrValue(vector<double> value)
{
    if (this->type != FLOATARR_ARG) throw std::string("Argument type is not FLOATARR");
    this->floatArrValue = value;
}

void ArgumentDesc::set_binValue(string value)
{
    if (this->type != BIN_ARG) throw std::string("Argument type is not BINARY");
    this->binValue = value;
}



string ArgumentDesc::getTypeAsString()
{ 
  switch (this->type)
  { case INT_ARG      : return "integer";
    case UNSIGNED_ARG : return "positive integer";
    case STRING_ARG   : return "string";
    case DOUBLE_ARG   : return "double";
    case BIN_ARG      : return "binary buffer";
    case LONG_ARG     : return "long";
    case FLOATARR_ARG : return "array of double";
  }
  return "parameter";
}


bool SwitchDesc::compare(string name)
{  return (this->switchString.compare(name)==0);
}

string  SwitchDesc::getSwitchName()
{  return this->switchString;
}

string  SwitchDesc::getDefaultValue()
{  return this->defaultStrValue;
}

void SwitchDesc::set_longValue(s64 value)
    { ArgumentDesc::set_longValue(value);
     set = true;
    }

void SwitchDesc::set_intValue(s64 value)
    { ArgumentDesc::set_intValue(value);
     set = true;
    }

void  SwitchDesc::set_unsignedValue(u64 value)
    {ArgumentDesc::set_unsignedValue(value);
     set = true;
    }

void  SwitchDesc::set_stringValue(string value)
    {ArgumentDesc::set_stringValue(value);
     set = true;
    }

void  SwitchDesc::set_doubleValue(double value)
    { ArgumentDesc::set_doubleValue(value);
      set = true;
    }

s64  SwitchDesc::get_longValue()
    { if (set) return ArgumentDesc::get_longValue(); 
     return atol(this->defaultStrValue.c_str());
    }

s64  SwitchDesc::get_intValue()
	{ if (set) return ArgumentDesc::get_intValue(); 
	  return atoi(this->defaultStrValue.c_str());
	}

u64  SwitchDesc::get_unsignedValue()
    { if (set) return ArgumentDesc::get_unsignedValue(); 
	  return atoi(this->defaultStrValue.c_str());
	}

string  SwitchDesc::get_stringValue(void)
    { if (set) return ArgumentDesc::get_stringValue(); 
	  return this->defaultStrValue;
	}

double  SwitchDesc::get_doubleValue(void)
    { if (set) return ArgumentDesc::get_doubleValue(); 
	  return atof(this->defaultStrValue.c_str());
	}


void YapiCommand::fatalError(string errmsg)
{
    printf("ERR: %s\n", errmsg.c_str());
    exit(-1);
}



string YapiCommand::execname(string argv0)
{
    string res = "";
    for (unsigned int i = 0; i < argv0.length(); i++) {
        unsigned char c = argv0[i];
        if ((c == '/') || (c == '\\')) res = "";
        else res += c;
    }
    return res;
}

string YapiCommand::helpformat(int marginFirstLine, int margin, int width, string data)
{  string result = "";
   string line = "";
   
   for (int i=0;i<marginFirstLine;i++) line=line+" ";
   for (size_t k=0;k<data.length();k++)
     {  if (data[k]!='\n')
		 { line = line+data[k];
           if (line.length()>(size_t)width)
		    { int n = (int)line.length()-1;
		      while ((n>1) && (line[n]!=' ')) n--;
		      int diff = ((int)line.length()-n);
		      k-=diff-1;
		      line = line.substr(0,n);
		      result = result+line+"\n";
		      line="";
		      for (int i=0;i<margin;i++) line=line+" ";
		   }
		 } else
         { result = result+line+"\n";
		   line="";
		   for (int i=0;i<margin;i++) line=line+" ";
		 }
     }

   if (line!="") result = result+line+"\n";
   return result;

}


void YapiCommand::showHelp(string execname)
{
    string extra = this->getMoreInfo();
    string fmt;

    printf("\nCommand %s:\n", this->getName().c_str());
    printf("%s\n",  helpformat(2,2,79,this->getDescription()).c_str());
    if (extra != "") {
		printf("%s\n", helpformat(2,2,79,extra).c_str());
    }
    printf("Usage:\n  ");
    if (this->requireTarget()) fmt = "%s <target> %s";
    else fmt = "%s %s";
    printf(fmt.c_str(), execname.c_str(), this->getName().c_str());
    vector<ArgumentDesc*> *params = this->getArgumentDesc();
    for (unsigned int i = 0; i < params->size(); i++) {
        if ((*params)[i]->optional) printf(" [");
        else printf(" <");
        printf("%s", (*params)[i]->name.c_str());
        if ((*params)[i]->optional)printf("]");
        else printf(">");
    }
    printf("\n\n");
    if (params->size() > 0) {
        bool intArgsUsed = false;
		bool intArrayUsed = false;

        printf("Arguments:\n");
        for (unsigned int i = 0; i < params->size(); i++) {
			string  line= (*params)[i]->name+" : "+ (*params)[i]->description;
            printf("%s", helpformat(2,6,79,line).c_str());
            if ((*params)[i]->type == INT_ARG || (*params)[i]->type == UNSIGNED_ARG || (*params)[i]->type ==  LONG_ARG) intArgsUsed = true;
            if ((*params)[i]->type == FLOATARR_ARG ) intArrayUsed = true;
		}
        printf("\n");
        if (intArgsUsed)
            printf("%s",helpformat(2,2,79,"Both decimal and hexadecimal syntax are accepted for integer parameters, hexadecimal syntax is 0xffffff or #ffffff or $ffffff").c_str());
		if (intArrayUsed)
            printf("%s",helpformat(2,2,79,"Array are defined as comma seperated list, without any space").c_str());
        printf("\n");
    }
    exit(0);
}

SwitchDesc* YapiCommand::findSwitch(string switchname, vector<SwitchDesc*>* list)
{   for (unsigned int i = 0; i < list->size(); i++)  
    { if ((*list)[i]->compare(switchname))
       { return (*list)[i];
       }
    }
    return NULL;
}


string YapiCommand::getClass()
{
    return "module";
}

vector<double>  YapiCommand::parseFloatArr(string data)
{
    vector<double> result = vector<double>();
    
    const char *start = data.c_str();
    const char *p = start;
    string num;
    
    while(*p) {
        start = p;
        while(*p && *p != ' ' && *p != ',') p++;
        num = string(start,p-start);
        result.push_back(atof(num.c_str()));
        start = p;
        while(*p == ' ' || *p == ',') p++;
    }
    
    return result;
}

// super atoi function wich understand both decimal
// and hexadecimal integer (with 0xFFFFFF , #FFFFFF, $FFFFFF)
// syntax

s64 YapiCommand::atoi(string data)
{
    if (data == "") return 0;
    int base = 10;
    int index = 0;
    int sign = 1;
    if (data[0] == '-') {
        index = 1;
        sign = -1;
    };
    if ((data[index] == '$') || (data[index] == '#')) {
        index++;
        base = 16;
    }
    else if (data.substr(index, 2) == "0x") {
        index += 2;
        base = 16;
    };
    s64 res = 0;
    for (unsigned int i = index; i < data.length(); i++) {
        unsigned char c = data[i];
        if ((c >= '0') && (c <= '9')) res = res * base + (c - '0');
        else if (base == 16) {
            if ((c >= 'a') && (c <= 'f')) res = res * base + (c - 'a') + 10;
            else if ((c >= 'A') && (c <= 'F')) res = res * base + (c - 'A') + 10;
        }
    }
    return res*sign;
}

//  makes sure a string is an integer
//  (both decimal and hexadeciamal are accepted)

bool YapiCommand::isInteger(string data)
{
    if (data == "") return 0;
    int base = 10;
    int index = 0;

    if (data[0] == '-') {
        index = 1;
    };
    if ((data[index] == '$') || (data[index] == '#')) {
        index++;
        base = 16;
    }
    else if (data.substr(index, 2) == "0x") {
        index += 2;
        base = 16;
    };
    for (unsigned i = index; i < data.length(); i++) {
        unsigned char c = data[i];
        if (!(((c >= '0') && (c <= '9'))
            || ((base == 16) &&
            (((c >= 'a') && (c <= 'f')) || ((c >= 'A') && (c <= 'F')))))) return false;

    }
    return true;
}

//  makes sure a string is an unsigned integer
//  (both decimal and hexadeciamal are accepted)

bool YapiCommand::isUnsigned(string data)
{
    if (data == "") return 0;
    if (data[0] == '-') return 0;
    return isInteger(data);
}

//  makes sure a string is a floating point value

bool YapiCommand::isDouble(string data)
{
    unsigned char c;
    int index = 0;
    if (data == "") return false;
    if (data[0] == '-') index = 1;

    for (unsigned int i = index; i < data.length(); i++) {
        c = data[i];
        if (!(((c >= '0') && (c <= '9')) || (c == '.'))) return false;
    }
    return true;
}

//  makes sure a string is a floating point value

bool YapiCommand::isFloatArr(string data)
{
    unsigned char c;
    int index = 0;
    if (data == "") return true;
    if (data[0] == '-') index = 1;
    
    for (unsigned int i = index; i < data.length(); i++) {
        c = data[i];
        if (!(((c >= '0') && (c <= '9')) || (c == '.') || (c == ','))) return false;
    }
    return true;
}

// simplifiy a string: convert all char to upcase and remove underscores

string YapiCommand::simplifyString(string s)
{
    string res = "";
    for (unsigned int i = 0; i < s.length(); i++) {
        unsigned char c = s[i];
        if ((c >= 'A') && (c <= 'Z')) res += c;
        else if ((c >= 'a') && (c <= 'z')) res += (c - 32);
        else if (c == '/') res += c;
		else if (c == '-') res += c;
    }
    return res;
}

vector<ArgumentDesc*>* YapiCommand::getArgumentDesc()
{
    vector<ArgumentDesc*>* res = new vector<ArgumentDesc*>();
    return res;
}

vector<SwitchDesc*>* YapiCommand::getSwitchDesc()
{  vector<SwitchDesc*>* res = new vector<SwitchDesc*>();
    return res;
}

bool YapiCommand::requireTarget(void)
{
    return true;
}

string YapiCommand::getMoreInfo(void)
{
    return "";
}

bool YapiCommand::MatchCmdLine(int argc, const char * argv[], int currentIndex, int* foundindex)
{
    if (this->requireTarget()) currentIndex++;
    while (currentIndex < argc) 
	 { string me = simplifyString(this->getName());
       string him = simplifyString(argv[currentIndex]);
	   if (me.compare(him) == 0) 
		    { *foundindex = currentIndex;
			  return true;
	        }
	   currentIndex++;
	 }
	
    return false;
}

bool YapiCommand::MissingTarget(int argc, const char * argv[], int currentIndex)
{
    if (!this->requireTarget()) return false;
    if (currentIndex >= argc) return false;
    string me = simplifyString(this->getName());
    string him = simplifyString(argv[currentIndex]);
    if (me.compare(him) == 0) return true;
    return false;
}

vector<SwitchDesc*>* YapiCommand::checkSwitches(int argc, const char * argv[], int startIndex,int endIndex)
{  vector<SwitchDesc*> *params = this->getSwitchDesc();

   if (startIndex > endIndex) return params;

   if ((params->size() == 0) && (startIndex <= endIndex))
        YapiCommand::fatalError("Command \"" + this->getName() + "\" takes no option after target.");

   for (int i =startIndex; i<endIndex;i++)
    { string switchname = argv[i];
      bool found= false;
      for (unsigned int j=0;(j<params->size()) && (!found) ;j++)
	   { string me  = simplifyString((*params)[j]->getSwitchName());
         if (me.compare(simplifyString(switchname))==0)
		  { found  = true;
		    if (i>=endIndex)
				YapiCommand::fatalError("Missing  \""+(*params)[j]->getTypeAsString()+"\" parameter after "+switchname); 
			i++;
			string value = argv[i];
			bool ok = this->checkArgumentValue((*params)[j], value);
            if (!ok) YapiCommand::fatalError(value + " is not a valid value for "+switchname);
		  }

	   }
	   if (!found)
	     YapiCommand::fatalError("Command \"" + this->getName() + "\" does not support \""+switchname+"\" switch");
    }

   return params;
}

bool YapiCommand::checkArgumentValue(ArgumentDesc* param, string currentValue)
 {  vector<string*>* allowedValues = param->acceptedValues;
    for (unsigned int j = 0; j < allowedValues->size(); j++) 
	  { if ((*allowedValues)[j]->compare("_LOGICAL") == 0) 
	      { if (param->type != STRING_ARG) throw std::string("_LOGICAL can be stored in STRING_ARG argument only");
                if (YAPI::CheckLogicalName(currentValue)) 
				  {  param->set_stringValue(currentValue);
                     return  true;
                  }
                 else YapiCommand::fatalError("\"" + currentValue + "\" is not a valid logical name");
           }
        else
        if ((*allowedValues)[j]->compare("_INT") == 0) 
		  { if (param->type != INT_ARG) throw std::string("_INT can be stored in INT_ARG argument only");
            if (YapiCommand::isInteger(currentValue)) 
			  { param->set_intValue(YapiCommand::atoi(currentValue.c_str()));
                return true;
              }
             else YapiCommand::fatalError(currentValue + " is not a valid integer value");
           }
         else
         if ((*allowedValues)[j]->compare("_LONG") == 0)
         { if (param->type != LONG_ARG) throw std::string("_LONG can be stored in LONG_ARG argument only");
             if (YapiCommand::isInteger(currentValue))
             { param->set_longValue(YapiCommand::atoi(currentValue.c_str()));
                 return true;
             }
             else YapiCommand::fatalError(currentValue + " is not a valid integer value");
         }
         else
         if ((*allowedValues)[j]->compare("_UNSIGNED") == 0)
		   {  if (param->type != UNSIGNED_ARG) throw std::string("_UNSIGNED can be stored in UNSIGNED_ARG argument only");
              if (YapiCommand::isUnsigned(currentValue)) 
			   {  param->set_unsignedValue(YapiCommand::atoi(currentValue.c_str()));
                  return true;
               }
              else YapiCommand::fatalError(currentValue + " is not a valid unsigned value");
           }
         else
         if ((*allowedValues)[j]->compare("_FLOATARR") == 0)
         { if (param->type != FLOATARR_ARG) throw std::string("_FLOATARR can be stored in FLOATARR_ARG argument only");
             if (YapiCommand::isFloatArr(currentValue))
             { param->set_floatArrValue(YapiCommand::parseFloatArr(currentValue.c_str()));
                 return true;
             }
             else YapiCommand::fatalError(currentValue + " is not a valid integer value");
         }
         else
         if ((*allowedValues)[j]->compare("_STRING") == 0)
		   {  if (param->type != STRING_ARG) throw std::string("_STRING can be stored in STRING_ARG argument only");
              param->set_stringValue(currentValue);
              return true;
           }
         else
         if ((*allowedValues)[j]->compare("_BIN") == 0) 
         {  if (param->type != BIN_ARG) throw std::string("_BIN can be stored in BIN_ARG argument only");
             if(currentValue.compare(0,5,"file:") == 0) {
                 currentValue = currentValue.substr(5,string::npos);
                 std::ifstream is(currentValue.c_str(),std::ifstream::binary);
                 if(!is) {
                     YapiCommand::fatalError("cannot open file " + currentValue);
                 } else {
                     is.seekg (0, is.end);
                     int length = (int)is.tellg();
                     is.seekg (0, is.beg);                     
                     char * buffer = new char [length];
                     is.read(buffer,length);
                     is.close();
                     currentValue = string(buffer,length);
                     delete[] buffer;
                 }
             } else if(currentValue.compare(0,4,"hex:") == 0) {
                 currentValue = currentValue.substr(4,string::npos);
                 int length = (int)(currentValue.size()/2);
                 char * buffer = new char [length];
                 for(int pos = 0; pos < length; pos++) {
                     buffer[pos] = (char)atoi("0x"+currentValue.substr(2*pos,2));
                 }
                 currentValue = string(buffer,length);
                 delete[] buffer;                 
             } else {
                 YapiCommand::fatalError(currentValue + " is not a valid binary buffer value (expected 'file:' or 'hex:')");
             }
             param->set_binValue(currentValue);
             return true;
         }
         else
         if ((*allowedValues)[j]->compare("_DOUBLE") == 0) 
		   {  if (param->type != DOUBLE_ARG) throw std::string("_DOUBLE can be stored in DOUBLE_ARG argument only");
              if (YapiCommand::isDouble(currentValue)) 
			   { param->set_doubleValue(atof(currentValue.c_str()));
                 return true;
               }
               else YapiCommand::fatalError(currentValue + " is not a valid floating point value");
           }
          else 
		   {  string v = *((*allowedValues)[j]);
              int p = (int)v.find("=", 0);
                if (p > 0) { 
		            string name = v.substr(0, p);
                    string value = v.substr(p + 1, v.length() - p - 1);
                    if (YapiCommand::simplifyString(name).compare(YapiCommand::simplifyString(currentValue)) == 0) {
                      switch (param->type) { 
                         case INT_ARG: param->set_intValue(YapiCommand::atoi(value.c_str()));
                             return true;
                             break;
                         case UNSIGNED_ARG: param->set_unsignedValue(YapiCommand::atoi(value.c_str()));
                             return true;
                             break;
                         case STRING_ARG: param->set_stringValue(value);
                             return true;
                             break;
                         case DOUBLE_ARG: param->set_doubleValue(atof(value.c_str()));
                             return true;
                             break;
                         case FLOATARR_ARG: param->set_floatArrValue(YapiCommand::parseFloatArr(value.c_str()));
                             return true;
                             break;
                         case BIN_ARG : param->set_binValue(value);
                             return true;
                             break;
                         case LONG_ARG : param->set_intValue(YapiCommand::atoi(value.c_str()));
                             return true;
                             break;
                      }
                   }
				}
	      }
                
	}
	return false;
 }



vector<ArgumentDesc*>* YapiCommand::checkArguments(int argc, const char * argv[], int currentIndex)
{
    int requiredArgs = 0;
    vector<ArgumentDesc*> *params = this->getArgumentDesc();
    if ((params->size() == 0) && (argc > currentIndex))
        YapiCommand::fatalError("Command \"" + this->getName() + "\" takes no argument.");

    if (argc - currentIndex > (int) params->size())
        YapiCommand::fatalError("Too many arguments for command \"" + this->getName() + "\"");

    for (unsigned i = 0; i < params->size(); i++)
        if (!(*params)[i]->optional) requiredArgs++;

    if (argc - currentIndex < requiredArgs)
        YapiCommand::fatalError("not enough arguments for command \"" + this->getName() + "\"");

    for (int i = 0; i < (int) argc - currentIndex; i++) 
	  { string currentValue = argv[currentIndex + i];
       
		bool ok = this->checkArgumentValue((*params)[i], currentValue);
        if (!ok) YapiCommand::fatalError(currentValue + " is not a valid value");
      }
		   
    return params;
}

s64 YapiCommand::getSwitchIntValue(vector<SwitchDesc*>* params,string name)
 {  for (unsigned i = 0; i < params->size(); i++)
     {  string him = (*params)[i]->getSwitchName();
		if (name.compare(him)==0)
          {   return (*params)[i]->get_intValue();
		    
          }
     }

    YapiCommand::fatalError("missing \"" + name + "\"  switch");
	return 0;
 }

/*
 * Output function, once replaceable tokens are expanded
 *
 * This function can handle redirection to a file if the
 * string is of the form: '@filename:line'
 */
void OutputLine(string line, string result)
{
    const char *p = line.c_str();
    if(*p == '@') {
        // output to file
        const char *fmt = p+1;
        while(*fmt && *fmt != ':') fmt++;
        string outfile = string(p+1, fmt-(p+1));
        string status;
        if(!*fmt) {
            line = result;
        } else {
            fmt++;
            line = string(fmt,line.size() - (fmt-p));
        }
        // insert newline as separator if the file exists and is
        // non-empty, but do not append it immediately after each line, as
        // the output might be binary data (eg. for apifunction download)
        FILE *f = fopen(outfile.c_str(), "r+b");
        if(f) {
            fseek(f, 0L, SEEK_END);
            if(ftell(f) > 0L) fwrite(NEWLINE, 1, strlen(NEWLINE), f);
            status = string("OK: append output to ")+outfile;
        } else {
            f = fopen(outfile.c_str(), "wb");
            if(f) status = string("OK: write output to new file ")+outfile;
        }
        if(f) {
            fwrite(line.c_str(), 1, line.size(), f);
            fclose(f);
        } else {
            status = string("ERR: failed to write to ")+outfile;
        }
        // output diagnostic message to standard output
        line = status;
    }
    
    // write to standard output (implicit add newline at the end)
    fwrite(line.c_str(), 1, line.size(), stdout);
    fwrite(NEWLINE, 1, strlen(NEWLINE), stdout);
}

string enumString(int numval, const char *enumdef[])
{
    if (numval >= 0) {
        int i;
        for (i = 0; i < numval; i++) {
            if (enumdef[i] == NULL) break;
        }
        if (enumdef[i] != NULL) return enumdef[i];
    }
    return "INVALID";
}

string bitString(const char *names, u64 val)
{
    char buffBits[70];
    char c, *p = buffBits;
    u16  n = 0;
    
    *p++ = '[';
    while((c = *names) != 0) {
        *p++ = ((val & 1) ? c : '.');
        val >>= 1;
        n++;
        names++;
    }
    *p++ = ']';
	*p = 0;
    
    return string(buffBits);
}

static void registerhubs(string arg)
{
    vector<string*> hublist;

    // split comma separated addresses
    hublist.push_back(new string());
    for (unsigned int i = 0; i < arg.length(); i++) {
        if (arg[i] == ',') hublist.push_back(new string());
        else *hublist[hublist.size() - 1] += arg.at(i);
    }

    // register hub adresses
    string errmsg = "";
    for (unsigned int i = 0; i < hublist.size(); i++) {
        if (YAPI::RegisterHub(*hublist[i], errmsg) != YAPI::SUCCESS)
            YapiCommand::fatalError("Cannot register " + *hublist[i] + ": " + errmsg);
    }

}

static bool cmdSortFct(YapiCommand* a, YapiCommand* b)
{
    return a->getName() < b->getName();
}

static void displayGlobalHelp(string execname, vector<YapiCommand*> cmdList)
{   
	SwitchList switchesMap;
	for (unsigned int i = 0; i < cmdList.size(); i++) 
	  { vector<SwitchDesc*>* s =   cmdList[i]->getSwitchDesc();
	    for (unsigned int j = 0; j < s->size(); j++) 
        { string key=  (*s)[j]->getSwitchName();
		  if (switchesMap.find(key)==switchesMap.end())
		   {   switchesMap.insert(std::pair<string, SwitchDesc*>(key, (*s)[j] ));		
		   }
	    }
	 }

	string switcheslist= "";
	string switchesDescription = "";
	for (SwitchList::iterator it=switchesMap.begin(); it!=switchesMap.end(); ++it)
	{  switcheslist=switcheslist+"["+it->second->getSwitchName().c_str()+" "+it->second->name.c_str()+ "] ";
	   string tmp ="  "+it->second->getSwitchName()+" "+it->second->name;
	   while (tmp.length()<16) tmp=tmp+" ";
	   string desc = it->second->description;
	   if  (it->second->optional) desc = desc + " Optional, default value is "+it->second->getDefaultValue(); 
	   unsigned int n=0;
	   int len=0;
	   string sep = "\n                  ";
	   int lastspace =0;
	   while (n<desc.length())
	   { char c = desc.at(n);
	     if  (c==' ') lastspace=n;
		 len++;
		 if (len>60)
		  { desc = desc.substr(0,lastspace)+sep+desc.substr(lastspace+1);
		    n=lastspace+(int)sep.length();
			len = 0;
		  }

		 n++;

	      
	   }

	   switchesDescription = tmp+": "+desc+"\n";
	}

    if (isModule())
        printf("Use %s to work with any Yoctopuce device.\n", execname.c_str());
    else
        printf("Use %s to work with any %s function on any Yoctopuce device.\n", execname.c_str(), YFunctionName().c_str());
    printf("Usage:\n\n");
    printf("%s [options] [target] %scommand [command arguments]\n\n", execname.c_str(),switcheslist.c_str());
    printf("\n");
    printf("options:\n");
    printf("  -r remoteAddr : Uses remote IP devices (or VirtalHub), instead of local USB.\n");
    printf("                  remoteAddr is a comma-separated list of IP name or addresses.\n");
    printf("  -f 'fmtString': Specify a custom format template for displaying output of\n");
    printf("                  executed commands. The default format template string is:\n");
    printf("                  'OK: [identifierOrName].[command] = [result].'\n");
    printf("                  For more details, please refer to the documentation.\n");
    printf("  -s            : Automatically call \"saveToFlash\"  on the matching module\n");
    printf("                  after command execution. This will save *all* changes\n");
    printf("                  previously made on the device (applies to set_* cmd only).\n");
    printf("  -d            : Enable Yocto-API debug mode\n");
    printf("%s",switchesDescription.c_str());

    printf("target:\n");
    if (isModule())
        printf("  A valid device serialNumber or logicalName.\n");
    else
        printf("  A valid \"serialNumber.logicalName\" pair for the %s function.\n", YFunctionName().c_str());
    printf("  \"any\" and \"all\" are also valid targets. Note that some commands do not\n");
    printf("  require a target\n");
    printf("\n");
    printf("Available commands (underscore and case-insensitive):\n");

    std::sort(cmdList.begin(), cmdList.end(), cmdSortFct);

    unsigned colHeight = (unsigned)((cmdList.size() + 2) / 3);
    for (unsigned int i = 0; i < colHeight; i++) {
        for (unsigned int j = 0; j < 3; j++) {
            if (j == 0)printf(" ");
            unsigned index = j * colHeight + i;
            if (index < cmdList.size()) {
                string n = cmdList[index]->getName();
                printf(" %s", n.c_str());
                for (unsigned k = (unsigned)n.length(); k < 25; k++) printf(" ");
                if (j == 2)printf("\n");
            }
            else printf("\n");
        }
    }
    printf("\n\n");
    printf("Type \"%s {command} /help\" for help about a specific command.\n", execname.c_str());
    exit(0);

}

static void log(const string& msg)
{
    printf("DBG: %s",msg.c_str());
}

int main(int argc, const char * argv[])
{
    vector<YapiCommand*> cmdList;
    string formatstring = "";
    bool autosave = false;

    int paramindex = 1;
    bool registered = false;

    // register specific commands
    RegisterCommands(&cmdList);

    // trap request for global help
    if (paramindex < argc) {
        string h = YapiCommand::simplifyString(argv[paramindex]);
        if (h.compare("/HELP") == 0) {
            displayGlobalHelp(YapiCommand::execname(argv[0]), cmdList);
            exit(0);
        }
    }

    // trap request for specific command help
    if (paramindex + 1 < argc) {
        string h = YapiCommand::simplifyString(argv[paramindex + 1]);
        if (h.compare("/HELP") == 0) {
            string cmd = YapiCommand::simplifyString(argv[paramindex]);
            for (unsigned int i = 0; i < cmdList.size(); i++)
                if (cmd.compare(YapiCommand::simplifyString(cmdList[i]->getName())) == 0)
                    cmdList[i]->showHelp(YapiCommand::execname(argv[0]));

		    // command no found, lets show global help
            printf("No help available for %s (unknow command)\n\n",cmd.c_str());
			displayGlobalHelp(YapiCommand::execname(argv[0]), cmdList);
            exit(0);
        }
    }


	// search for -d flag, this must be done before the registerhub
	while ((paramindex < argc) && (argv[paramindex][0] == '-')) { 
	 if (strcmp(argv[paramindex], "-d") == 0)
		 YAPI::RegisterLogFunction(log);
	       paramindex++;
    }

    paramindex = 1;

    while ((paramindex < argc) && (argv[paramindex][0] == '-')) { // search for register hub fla
        if ((strcmp(argv[paramindex], "-r") == 0) && (paramindex < argc - 1)) {
            registerhubs(argv[paramindex + 1]);
            registered = true;
            paramindex++;
        } else if ((strcmp(argv[paramindex], "-f") == 0) && (paramindex < argc - 1)) {
            formatstring = string(argv[paramindex + 1]);
            paramindex++;
        } else if (strcmp(argv[paramindex], "-s") == 0) {
            autosave = true;
        }
        paramindex++;
    }

    // not register flag found, lets use default USB 
    if (!registered) registerhubs("usb");

  

    // looking for the right command to execute
    for (unsigned int i = 0; i < cmdList.size(); i++) {
		int foundindex=-1;
        if (cmdList[i]->MatchCmdLine(argc, argv, paramindex, &foundindex)) 
		 {  string target = "";
            if (cmdList[i]->requireTarget()) target = argv[paramindex++];

            // trap help request in cmd arguments
            for (int j = foundindex + 1; j < argc; j++) {
                if (YapiCommand::simplifyString(argv[j]) == "/HELP") {
                    cmdList[i]->showHelp(YapiCommand::execname(argv[0]));
                }
            } 

            vector<SwitchDesc*>* switches = cmdList[i]->checkSwitches(argc, argv, paramindex,foundindex-1);
            vector<ArgumentDesc*>*   args = cmdList[i]->checkArguments(argc, argv, foundindex+1);
            vector<YModule*> modulelist;
            try {
                cmdList[i]->execute(target, &modulelist, formatstring, args,switches);
            }
            catch (exception& e) {
                YapiCommand::fatalError("internal error: " + (string) e.what());
            }
            // calls SaveToFlash if needed 
            if (autosave) {
                if (cmdList[i]->getName().substr(0, 4) == "set_") {
                    if (cmdList[i]->getName().substr(0, 4) != "set_beacon") { // little ugly exception
                        for (unsigned j = 0; j < modulelist.size(); j++) {
                            if (modulelist[j]->isOnline()) {
                                modulelist[j]->saveToFlash();
                                PrintResult(formatstring, "saveToFlash", modulelist[j], "", false);
                            } else {
                                YapiCommand::fatalError(modulelist[j]->get_serialNumber()+".saveToFlash failed (module went offline)");
                            }
                        }
                    }
                }
            }
            YAPI::FreeAPI();
            exit(0);
        }
        else
            if (cmdList[i]->MissingTarget(argc, argv, paramindex)) {
            YapiCommand::fatalError(" command " + (string) argv[paramindex] + ": missing target, use  \"" + YapiCommand::execname(argv[0]) + " " + (string) argv[paramindex] + " /help\" for help.");
        }
    }

    string help = "\nUse \"" + YapiCommand::execname(argv[0]) + " /help\" for help.";
    if (paramindex>=argc) YapiCommand::fatalError("missing command." + help);
    else if (paramindex+1>=argc) YapiCommand::fatalError("unknown command:" + (string) argv[paramindex] + help);
    else YapiCommand::fatalError("unknown command:" + (string) argv[paramindex+1] + help);


}

