/*********************************************************************
 *
 * $Id: cmdline_common.cpp 7646 2012-09-13 15:05:29Z mvuilleu $
 *
 * declaration of cmdline  API generic class & functions
 *
 * - - - - - - - - - License information: - - - - - - - - -
 *
 * Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
 *
 * 1) If you have obtained this file from www.yoctopuce.com,
 *    Yoctopuce Sarl licenses to you (hereafter Licensee) the
 *    right to use, modify, copy, and integrate this source file
 *    into your own solution for the sole purpose of interfacing
 *    a Yoctopuce product with Licensee's solution.
 *
 *    The use of this file and all relationship between Yoctopuce
 *    and Licensee are governed by Yoctopuce General Terms and
 *    Conditions.
 *
 *    THE SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT
 *    WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
 *    WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
 *    FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
 *    EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
 *    INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
 *    COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
 *    SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
 *    LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
 *    CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
 *    BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
 *    WARRANTY, OR OTHERWISE.
 *
 * 2) If your intent is not to interface with Yoctopuce products,
 *    you are not entitled to use, read or create any derived
 *    material from this source file.
 *
 *********************************************************************/

#include <string>
#include <vector>

#include "yocto_api.h"
#include "yocto_files.h"
#include "yocto_wireless.h"
#include "yocto_display.h"

using namespace std;

// snprintf is called sprintf_s in VisualC++
#ifdef _MSC_VER
#define snprintf sprintf_s
#endif

/*
 * handling of commands arguements. 
 */
typedef enum {
    INT_ARG, UNSIGNED_ARG, STRING_ARG, DOUBLE_ARG ,BIN_ARG, LONG_ARG, FLOATARR_ARG
} argTypes;

class ArgumentDesc {
public:
    ArgumentDesc(argTypes type, string name, string descrption, string acceptedValues, bool optional);
    ~ArgumentDesc();
    string name;
    string description;
    argTypes type;
    vector<string*>* acceptedValues;
    bool optional;

	string getTypeAsString();

    s64 intValue;
    u64 unsignedValue;
    string stringValue;
    double doubleValue;
    vector<double> floatArrValue;
    string binValue;

    virtual s64 get_longValue();
    virtual s64 get_intValue();
    virtual u64 get_unsignedValue();
    virtual string get_stringValue(void);
    virtual double get_doubleValue(void);
    virtual string get_binValue(void);
    virtual vector<double> get_floatArrValue(void);
    string to_str(void);

    virtual void set_longValue(s64 value);
    virtual void set_intValue(s64 value);
    virtual void set_unsignedValue(u64 value);
    virtual void set_stringValue(string value);
    virtual void set_doubleValue(double value);
    virtual void set_floatArrValue(vector<double> value);
    virtual void set_binValue(string value);
};




class SwitchDesc: public ArgumentDesc
{ 
	 string switchString;
	 string defaultStrValue;
	 bool  set;
public: 
   SwitchDesc(string switchstr, argTypes type, string name, string descrption, string acceptedValues, bool optional,string defaultValue):
		ArgumentDesc( type,  name,  descrption,  acceptedValues,  optional),
		switchString(switchstr) 
		{ set=false;
		  defaultStrValue=defaultValue;
		};
   bool compare(string name);

   void set_longValue(s64 value);
   void set_intValue(s64 value);
   void set_unsignedValue(u64 value);
   void set_stringValue(string value);
   void set_doubleValue(double value);

   s64 get_longValue();
   s64 get_intValue();
   u64 get_unsignedValue();
   string get_stringValue(void);
   double get_doubleValue(void);

   string getSwitchName();
   string getDefaultValue();
 

};

class YapiCommand {
public:
    static void fatalError(string errmsg);
    static string execname(string argv0);

    virtual string getName(void) = 0;
    virtual string getDescription(void) = 0;
    virtual string getMoreInfo(void);
    virtual bool requireTarget(void);
    virtual vector<ArgumentDesc*>* getArgumentDesc(void);
	virtual vector<SwitchDesc*>*  getSwitchDesc(void);
    void showHelp(string execname);
	string helpformat(int marginFirstLine, int margin, int width, string data);
    static string getClass();
    static bool isInteger(string data);
    static bool isUnsigned(string data);
    static bool isDouble(string data);
    static bool isFloatArr(string data);
	
    s64 getSwitchIntValue(vector<SwitchDesc*>* params,string name);


	static SwitchDesc* findSwitch(string switchname, vector<SwitchDesc*>*);
	bool checkArgumentValue(ArgumentDesc* param, string currentValue);
    static vector<double> parseFloatArr(string data);
    static s64 atoi(string data);
    static string simplifyString(string s);
    bool MatchCmdLine(int argc, const char * argv[], int currentIndex,int* foundindex);
    bool MissingTarget(int argc, const char * argv[], int currentIndex);
    vector<ArgumentDesc*>* checkArguments(int argc, const char * argv[], int currentIndex);
	vector<SwitchDesc*>* checkSwitches(int argc, const char * argv[], int startIndex,int endIndex);
    virtual void execute(string target, vector<YModule*> *modulelist, string resultformat, vector<ArgumentDesc*>* desc , vector<SwitchDesc*>* switches) = 0;
	
};

void OutputLine(string line, string result);
string enumString(int numval, const char *enumdef[]);
string bitString(const char *names, u64 val);
void RegisterCommands(vector<YapiCommand*>* cmdList);
string YFunctionName(void);
bool isModule(void);
int main(int argc, const char * argv[]);

#ifdef __WIN32__
#define NEWLINE "\r\n"
#else
#define NEWLINE "\n"
#endif

/*
 * template function for module/function description 
 */
template <class yFct> string UserFriendlyDescription(yFct *f) {
    if (isModule()) {
        string logical = f->get_logicalName();
        if (logical != "") return f->get_logicalName();
        return f->get_module()->get_serialNumber();
    }

    string hwid = f->get_hardwareId();
    unsigned int p = (unsigned)hwid.find(".");
    string res = "";

    string moduleLogical = f->get_module()->get_logicalName();
    string fctLogical = f->get_logicalName();
    string moduleSerial = hwid.substr(0, p);
    string fctHwdName = hwid.substr(p + 1, hwid.length() - p);

    return (moduleLogical != "" ? moduleLogical : moduleSerial) + "." + (fctLogical != "" ? fctLogical : fctHwdName);
}

/*
 * template function for displaying result
 */

template <class yFct> void PrintResult(string format, string command, yFct* target, string result, bool is_get)
{
    const char *fmt = format.c_str();
    if(!*fmt) {
        if(is_get) fmt = "OK: [identifierOrName].[command] = [result].";
        else       fmt = "OK: [identifierOrName].[command] executed.";
    }
    const char *p = fmt;
    string line;
    while(*p) {
        while(*p && *p != '[') p++;
        if(p > fmt) line += string(fmt,p-fmt);
        if(!*p) break;
        fmt = ++p; // skip [
        while(*p && *p != '[' && *p != ']') p++;
        if(*p != ']') {
            line += '[';
            // closing brace not found, restart after '['
            p = fmt;
        } else {
            string word = string(fmt, p-fmt);
            p++;
            if(word == "identifierOrName") {
                line += UserFriendlyDescription<yFct>(target);
            } else if(word == "hardwareId") {
                line += target->get_hardwareId();
            } else if(word == "functionId") {
                line += target->get_functionId();
            } else if(word == "logicalName") {
                line += target->get_logicalName();
            } else if(word == "friendlyName") {
                line += target->get_friendlyName();
            } else if(word == "serialNumber") {
                line += target->get_module()->get_serialNumber();
            } else if(word == "moduleName") {
                line += target->get_module()->get_logicalName();
            } else if(word == "command") {
                line += command;
            } else if(word == "result") {
                line += result;
            } else {
                // unknown tag, restart after initial '['
                line += '[';
                p = fmt;
            }
        }
        fmt = p;
    }
    OutputLine(line, result);
}

template <class yFct> void PrintCmdResult(string format, string command, yFct* target)
{
    PrintResult<yFct>(format, command, target, "", false);
}

/*
 * specific display functions
 */

template <class yFct> void PrintResult(string format, string command, yFct* target, int result, bool is_get)
{
    char buffer[32];
    sprintf(buffer, "%d", result);
    PrintResult<yFct>(format, command, target, string(buffer), is_get);
}

template <class yFct> void PrintResult(string format, string command, yFct* target, unsigned result, bool is_get)
{
    char buffer[32];
    sprintf(buffer, "%u", result);
    PrintResult<yFct>(format, command, target, string(buffer), is_get);
}

template <class yFct> void PrintResult(string format, string command, yFct* target, long result, bool is_get)
{
    char buffer[32];
    sprintf(buffer, "%lu", result);
    PrintResult<yFct>(format, command, target, string(buffer), is_get);
}

template <class yFct> void PrintResult(string format, string command, yFct* target, double result, bool is_get)
{
    char buffer[32];
    sprintf(buffer, "%.4f", result);
    PrintResult<yFct>(format, command, target, string(buffer), true);
}

template <class yFct> void PrintResult(string format, string command, yFct* target, YMeasure& result, bool is_get)
{
    char buffer[64];
	double delta = result.get_endTimeUTC()-result.get_startTimeUTC();

	if  (delta<=0.09)
	sprintf(buffer, "%.2f;%.4f;%.4f;%.4f", result.get_endTimeUTC(),
            result.get_minValue(), result.get_averageValue(), result.get_maxValue());
    else
	if  (delta<=0.9)
	sprintf(buffer, "%.1f;%.4f;%.4f;%.4f", result.get_endTimeUTC(),
            result.get_minValue(), result.get_averageValue(), result.get_maxValue());
    else
    sprintf(buffer, "%u;%.4f;%.4f;%.4f", (u32)result.get_endTimeUTC(),
            result.get_minValue(), result.get_averageValue(), result.get_maxValue());
    PrintResult<yFct>(format, command, target, string(buffer), true);
}

template <class yFct> void PrintResult(string format, string command, yFct* target, vector<YMeasure> result, bool is_get)
{
    for(size_t i = 0; i < result.size(); i++) {
        PrintResult<yFct>(format, command, target, result.at(i), true);
    }
}

template <class yFct> void PrintResult(string format, string command, yFct* target, vector<double> result, bool is_get)
{
    char buffer[32];
    string value;

    if(result.size() == 0) {
        PrintResult<yFct>(format, command, target, "", true);
        return;
    }
    sprintf(buffer, "%.4f", result.at(0));
    value = string(buffer);
    for(size_t i = 1; i < result.size(); i++) {
        sprintf(buffer, "%.4f", result.at(i));
        value += string(",")+string(buffer);
    }
    PrintResult<yFct>(format, command, target, value, true);
}

template <class yFct> void PrintResult(string format, string command, yFct* target, YDataSet& result, bool is_get)
{
    PrintResult<yFct>(format, command, target, "unixTime;minValue;averageValue;maxValue", true);
    size_t written = 0;
    int progress = 0;
    while(progress < 100) {
        progress = result.loadMore();
        vector<YMeasure> measures = result.get_measures();
        vector<YMeasure> added;
        while(written < measures.size()) {
            added.push_back(measures.at(written++));
        }
        if(added.size() > 0) {
            PrintResult<yFct>(format, command, target, added, true);
        }
    }
}

template <class yFct> void PrintResult(string format, string command, yFct* target, vector<YDataSet> result, bool is_get)
{
    for(size_t i = 0; i < result.size(); i++) {
        PrintResult<yFct>(format, command, target, result.at(i), true);
    }
}

template <class yFct> void PrintResult(string format, string command, yFct* target, YFileRecord& result, bool is_get)
{
    char buffer[32];
    sprintf(buffer, ";%d;%08X", result.get_size(), result.get_crc());
    if(format=="") format = "OK;[friendlyName];[command];[result]";
    PrintResult<yFct>(format, command, target, result.get_name()+string(buffer), true);
}

template <class yFct> void PrintResult(string format, string command, yFct* target, vector<YFileRecord> result, bool is_get)
{
    PrintResult<yFct>(format, command, target, "name;size;crc", true);
    for(size_t i = 0; i < result.size(); i++) {
        PrintResult<yFct>(format, command, target, result.at(i), true);
    }
}

template <class yFct> void PrintResult(string format, string command, yFct* target, YWlanRecord& result, bool is_get)
{
    char buffer[64];
    sprintf(buffer, ";%d;%s;%d%%", result.get_channel(), result.get_security().c_str(), result.get_linkQuality());
    if(format=="") format = "OK;[friendlyName];[command];[result]";
    PrintResult<yFct>(format, command, target, result.get_ssid()+string(buffer), true);
}

template <class yFct> void PrintResult(string format, string command, yFct* target, vector<YWlanRecord> result, bool is_get)
{
    PrintResult<yFct>(format, command, target, "ssid;channel;security;linkQuality", true);
    for(size_t i = 0; i < result.size(); i++) {
        PrintResult<yFct>(format, command, target, result.at(i), true);
    }
}

/*
 * template function for inventory
 */

template <class yFct> vector<yFct*> *enumerateTargets(string target, vector<YModule*> *modulelist) {
    vector<yFct*>* res = new vector<yFct*>();
    yFct* m;
    string simplified = YapiCommand::simplifyString(target);

    //  any or every ones
    if ((simplified.compare("ANY") == 0) || (simplified.compare("ALL") == 0)) {
        m = yFct::First();
        if (m == NULL)
            YapiCommand::fatalError("No " + YapiCommand::getClass() + " found");
        res->push_back(m);
        modulelist->push_back(m->get_module());
        if (simplified.compare("ANY") == 0) return res;
        do {
            m = m->next();
            if (m != NULL)
                if (m->isOnline()) {
                    res->push_back(m);
                    bool found = 0;
                    YModule* HostModule = m->get_module();
                    string HostModuleSerial = HostModule->get_serialNumber();
                    for (unsigned int i = 0; i < modulelist->size(); i++)
                        if ((*modulelist)[i]->get_serialNumber() == HostModuleSerial)
                            found = true;
                    if (!found) modulelist->push_back(HostModule);
                }



        }        while (m != NULL);
        return res;
    }

    // specific one
    m = yFct::Find(target);
    if (!m->isOnline())
        YapiCommand::fatalError("No online " + YapiCommand::getClass() + " named " + target);

    // fix : 
    m = isModule() ? yFct::Find(m->get_module()->get_serialNumber()) : yFct::Find(m->get_hardwareId());
    if (!m->isOnline())
        YapiCommand::fatalError("No online " + YapiCommand::getClass() + " named " + target);

    res->push_back(m);
    modulelist->push_back(m->get_module());

    return res;
}

/*
 * template class for inventory command
 */
template <class yFct> class InventoryCmd : public YapiCommand {
public:

    string getDescription() {
        if (isModule()) return "List all available modules Module, give both serial number and logical name if available.";
        else return "Display all available " + (string) YFunctionName() + " functions, give both serial number and logical name if available.";
    }

    string getName() {
        return "inventory";
    }

    bool requireTarget() {
        return false;
    }

    void execute(string target, vector<YModule*> *modulelist, string resultformat, vector<ArgumentDesc*>* args, vector<SwitchDesc*>* switches) {
        vector<yFct*>* list = enumerateTargets<yFct > ("ALL", modulelist);

        for (unsigned int i = 0; i < list->size(); i++) {
            string ms = (*list)[i]->get_module()->get_serialNumber();
            string ml = (*list)[i]->get_module()->get_logicalName();


            if (isModule()) {
                printf("%-20s ", ms.c_str());
                if (ml != "")printf("(%s)", ml.c_str());
                printf("\n");
            } else {
                string res = "";
                string l = (*list)[i]->get_logicalName();
                string hwid = (*list)[i]->get_hardwareId();
                unsigned int p = (unsigned)hwid.find(".");
                string h = hwid.substr(p + 1, hwid.length() - p);

                printf("%s.%s", ms.c_str(), h.c_str());
                if ((l != "") || (ml != "")) {
                    if (l == "") l = h;
                    if (ml == "") ml = ms;
                    printf("  (%s.%s)", ml.c_str(), l.c_str());
                }
                printf("\n");
            }

        }
    }

};

/*
 * template class for version command
 */
template <class yFct> class versionCmd : public YapiCommand {
public:

    string getDescription() {
        return "Shows Yocto-API version.";
    }

    string getName() {
        return "version";
    }

    bool requireTarget() {
        return false;
    }

    void execute(string target, vector<YModule*> *modulelist, string resultformat, vector<ArgumentDesc*>* args, vector<SwitchDesc*>* switches) {
        printf("OK: version = %s\n", YAPI::GetAPIVersion().c_str());

    }

};

/*
 * template class for describe command
 */
template <class yFct> class DescribeCmd : public YapiCommand {
public:

    string getDescription() {
        if (isModule()) return "Display info about  modules";
        else return "Display  info about  " + (string) YFunctionName() + " functions.";
    }

    string getName() {
        return "describe";
    }

    void execute(string target, vector<YModule*> *modulelist, string resultformat, vector<ArgumentDesc*>* args, vector<SwitchDesc*>* switches) {
        char buffer[256];
        vector<yFct*>* list = enumerateTargets<yFct > (target, modulelist);
        for (unsigned int i = 0; i < list->size(); i++) {
            string l = (*list)[i]->get_module()->get_logicalName().c_str();
            printf("%-30s : %s\n", "Module serial number", (*list)[i]->get_module()->get_serialNumber().c_str());
            printf("%-30s : %s\n", "Module logical name", l != "" ? l.c_str() : "(not set)");
            printf("%-30s : %s\n", "Module product name", (*list)[i]->get_module()->get_productName().c_str());
            printf("%-30s : 0x%x\n", "Module product ID", (*list)[i]->get_module()->get_productId());
            printf("%-30s : %s\n", "Module firmware", (*list)[i]->get_module()->get_firmwareRelease().c_str());
            string uptime = "";
            unsigned long up = (*list)[i]->get_module()->get_upTime() / 1000;
            if (up > 86400) {
                snprintf(buffer, 256, "%dd ", (int) (up / 86400));
                uptime = string(buffer);
                up = up % 86400;
            }
            if ((up > 3600) || (uptime != "")) {
                snprintf(buffer, 256, "%dh ", (int) (up / 3600));
                uptime += string(buffer);
                up = up % 3600;
            }
            if ((up > 60) || (uptime != "")) {
                snprintf(buffer, 256, "%dm ", (int) (up / 60));
                uptime += string(buffer);
                up = up % 60;
            }
            snprintf(buffer, 256, "%ds", (int) up);
            uptime += string(buffer);



            printf("%-30s : %s\n", "Module uptime", uptime.c_str());

            if (!isModule()) {
                string hwid = (*list)[i]->get_hardwareId();
                unsigned int p = (unsigned)hwid.find(".");
                string l = (*list)[i]->get_logicalName().c_str();
                string fctHwdName = hwid.substr(p + 1, hwid.length() - p);
                printf("%-30s : %s\n", "Function hardware name", fctHwdName.c_str());
                printf("%-30s : %s\n", "Function logical name", l != "" ? l.c_str() : "(not set)");

            }
            printf("\n");
        }
    }
};

/*
 * template function for description for registering
 * generic functions available in all cmd lines
 */

template <class yFct> void RegisterGenericFunctions(vector<YapiCommand*>* cmdList) {
    cmdList->push_back(new InventoryCmd<yFct > ());
    cmdList->push_back(new DescribeCmd<yFct > ());
    cmdList->push_back(new versionCmd<yFct > ());
}

typedef map<string, SwitchDesc* > SwitchList;
